########################################
# git.py
# port from https://github.com/SirAnthony/slpp
# Modifier zrong(zengrong.net)
#
# Creation 2014-10-23
# Last Editing 2015-06-14
########################################
"""
.. module:: lua
   :platform: Unix, Windows
   :synopsis: lua-python 解析模块。

.. moduleauthor:: zrong(zengrong.net)

"""
import re
from rookout.base import read_file

ERRORS = {
    'unexp_table': u'Unexpected structure while parsing Lua string.',
    'unexp_end_string': u'Unexpected end of string while parsing Lua string.',
    'unexp_end_table': u'Unexpected end of table while parsing Lua string.',
    'mfnumber_minus': u'Malformed number (no digits after initial minus).',
    'mfnumber_dec_point': u'Malformed number (no digits after decimal point).',
    'mfnumber_sci': u'Malformed number (bad scientific format).',
}


class Lua:
    """进行 lua 标准 table 和 python 数据之间的转换。使用 python re 模块进行解析。"""

    def __init__(self):
        self.text = ''
        self.ch = ''
        self.at = 0
        self.len = 0
        self.depth = 0
        self.space = re.compile('\s', re.M)
        self.alnum = re.compile('\w', re.M)
        self.newline = '\n'
        self.tab = '\t'

    def decode(self, text):
        if not text or type(text) is not str:
            return
        #FIXME: only short comments removed
        reg = re.compile('--.*$', re.M)
        text = reg.sub('', text, 0)
        tstart = text.find('{')
        tend = text.rfind('}')
        if tstart < 0 and tend < 0:
            self.text = text
        elif tstart < 0 or tend < 0:
            raise TypeError(ERRORS['unexp_table'])
        else:
            self.text = text[tstart:tend+1]
        self.at, self.ch, self.depth = 0, '', 0
        self.len = len(self.text)
        self.next_chr()
        result = self.value()
        return result

    def encode(self, obj):
        if not obj:
            return
        self.depth = 0
        return self.__encode(obj)

    def __encode(self, obj):
        s = ''
        tab = self.tab
        newline = self.newline
        tp = type(obj)
        if tp is str:
            s += '"%s"' % obj.replace(r'"', r'\"')
        elif tp in [int, float, complex]:
            s += str(obj)
        elif tp is bool:
            s += str(obj).lower()
        elif tp in [list, tuple, dict]:
            self.depth += 1
            if len(obj) == 0 or ( tp is not dict and len(list(filter(
                    lambda x:  type(x) in (int,  float) \
                    or (type(x) is str and len(x) < 10),  obj
                ))) == len(obj) ):
                newline = tab = ''
            dp = tab * self.depth
            s += "%s{%s" % (tab * (self.depth - 2), newline)
            if tp is dict:
                s += (',%s' % newline).join(
                    [self.__encode(v) if type(k) is int \
                        else dp + '["%s"] = %s' % (k, self.__encode(v)) \
                        for k, v in obj.items()
                    ])
            else:
                s += (',%s' % newline).join(
                    [dp + self.__encode(el) for el in obj])
            self.depth -= 1
            s += "%s%s}" % (newline, tab * self.depth)
        return s

    def white(self):
        while self.ch:
            if self.space.match(self.ch):
                self.next_chr()
            else:
                break

    def next_chr(self):
        if self.at >= self.len:
            self.ch = None
            return None
        self.ch = self.text[self.at]
        self.at += 1
        return True

    def value(self):
        self.white()
        if not self.ch:
            return
        if self.ch == '{':
            return self.object()
        if self.ch == "[":
            self.next_chr()
        if self.ch in ['"',  "'",  '[']:
            return self.string(self.ch)
        if self.ch.isdigit() or self.ch == '-':
            return self.number()
        return self.word()

    def string(self, end=None):
        s = ''
        start = self.ch
        if end == '[':
            end = ']'
        if start in ['"',  "'",  '[']:
            while self.next_chr():
                if self.ch == end:
                    self.next_chr()
                    if start != "[" or self.ch == ']':
                        return s
                if self.ch == '\\' and start == end:
                    self.next_chr()
                    if self.ch != end:
                        s += '\\'
                s += self.ch
        slog.error(ERRORS['unexp_end_string'])

    def object(self):
        o = {}
        k = ''
        idx = 0
        numeric_keys = False
        self.depth += 1
        self.next_chr()
        self.white()
        if self.ch and self.ch == '}':
            self.depth -= 1
            self.next_chr()
            return o #Exit here
        else:
            while self.ch:
                self.white()
                if self.ch == '{':
                    o[idx] = self.object()
                    idx += 1
                    continue
                elif self.ch == '}':
                    self.depth -= 1
                    self.next_chr()
                    if k:
                       o[idx] = k
                    if not numeric_keys and len([ key for key in o if type(key) in (str,  float,  bool,  tuple)]) == 0:
                        ar = []
                        for key in o:
                           ar.insert(key, o[key])
                        o = ar
                    return o #or here
                else:
                    if self.ch == ',':
                        self.next_chr()
                        continue
                    else:
                        k = self.value()
                        if self.ch == ']':
                            numeric_keys = True
                            self.next_chr()
                    self.white()
                    if self.ch == '=':
                        self.next_chr()
                        self.white()
                        o[k] = self.value()
                        idx += 1
                        k = ''
                    elif self.ch == ',':
                        self.next_chr()
                        self.white()
                        o[idx] = k
                        idx += 1
                        k = ''
        slog.error(ERRORS['unexp_end_table']) #Bad exit here

    def word(self):
        s = ''
        if self.ch != '\n':
          s = self.ch
        while self.next_chr():
            if self.alnum.match(self.ch):
                s += self.ch
            else:
                if re.match('^true$', s, re.I):
                    return True
                elif re.match('^false$', s, re.I):
                    return False
                elif s == 'nil':
                    return None
                return str(s)

    def number(self):
        def next_digit(err):
            n = self.ch
            self.next_chr()
            if not self.ch or not self.ch.isdigit():
                raise TypeError(err)
            return n
        n = ''
        try:
            if self.ch == '-':
                n += next_digit(ERRORS['mfnumber_minus'])
            n += self.digit()
            if n == '0' and self.ch in ['x', 'X']:
                n += self.ch
                self.next_chr()
                n += self.hex()
            else:
                if self.ch and self.ch == '.':
                    n += next_digit(ERRORS['mfnumber_dec_point'])
                    n += self.digit()
                if self.ch and self.ch in ['e', 'E']:
                    n += self.ch
                    self.next_chr()
                    if not self.ch or self.ch not in ('+', '-'):
                        raise TypeError(ERRORS['mfnumber_sci'])
                    n += next_digit(ERRORS['mfnumber_sci'])
                    n += self.digit()
        except TypeError as e:
            slog.error(e)
            return 0
        try:
            return int(n, 0)
        except:
            pass
        return float(n)

    def digit(self):
        n = ''
        while self.ch and self.ch.isdigit():
            n += self.ch
            self.next_chr()
        return n

    def hex(self):
        n = ''
        while self.ch and \
            (self.ch in 'ABCDEFabcdef' or self.ch.isdigit()):
            n += self.ch
            self.next_chr()
        return n


lua = Lua()
"""一个默认的 lua 模块"""

def decode_file(luafile):
    """将 lua文件 解析成 python 对象。
    将 luafile 解析成字符串，然后调用 :func:`rookout.lua.decode()` 。

    :param str luafile: lua文件路径。
    :return: python 对象
    :raise: :class:`TypeError`

    """
    luastr = read_file(luafile)
    return decode(luastr)

def decode(text):
    """将 lua 解析成 python 对象。
    text 可以是标准的 lua 字符串、或者直接是一个对象。

    下面的代码都能够通过测试：

    整数和浮点数：

    >>> assert lua.decode('3') == 3
    >>> assert lua.decode('4.1') == 4.1

    负浮点数：

    >>> assert lua.decode('-0.45') == -0.45

    科学计数法：

    >>> assert lua.decode('3e-7') == 3e-7
    >>> assert lua.decode('-3.23e+17') == -3.23e+17

    十六进制：

    >>> assert lua.decode('0x3a') == 0x3a

    字符串转义：

    >>> assert lua.decode(r"'test\'s string'") == "test's string"

    table：

    >>> data = '{ array = { 65, 23, 5 }, dict = { string = "value", array = { 3, 6, 4}, mixed = { 43, 54.3, false, string = "value", 9 } } }'
    >>> d = lua.decode(data)
    >>> # 下面是伪代码
    >>> d == lua.decode(lua.encode(d))

    table 中花括号之外的内容会被自动删除：

    >>> data = 'local data = {a = 1, b = {3, 4}} return data'
    >>> print(lua.decode(data))
    >>> {"a":1, {"b":[3,4]}}

    :param str text: lua字符串
    :return: python 对象
    :raise: :class:`TypeError`

    """
    return lua.decode(text)

def encode(obj):
    """将 python 对象解析成 lua 字符串。

    :param object obj: 一个 python 对象。
    :return: 一个 lua 格式的字符串。

    """
    return lua.encode(obj)




########################################
# git.py
# Author zrong(zengrong.net)
#
# Creation 2014-10-23
# Last Editing 2015-06-14
########################################
"""
.. module:: git
   :platform: Unix, Windows
   :synopsis: git 通用操作。

.. moduleauthor:: zrong(zengrong.net)

"""

import os
import subprocess
from rookout import slog


def call(path, *args):
    """使用 subprocess.check_output 调用 git 命令。

    :param str path: git 仓库文件夹路径。
    :param \*args: git 的附加参数。
    :returns: 错误代码和调用结果。
    :rtype: int
    :rtype: string git 返回的信息，若执行出错则为错误信息。

    """
    returncode = 0
    output = None
    try:
        output = subprocess.check_output(get_args(path, *args), 
                stderr=subprocess.STDOUT,
                universal_newlines=True)
    except subprocess.CalledProcessError as err:
        returncode = err.returncode
        output = err.output
    return returncode, output

def get_args(path, *args, bare=False):
    """获取可被 subprogress 执行的 git 参数 list。

    :param str path: git 仓库文件夹路径。
    :param \*args: git 的附加参数。
    :param bare: 是否视为 bare 库

    """
    base = [ 'git' ]
    if path:
        if bare:
            base.append('--bare')
            base.append("--git-dir="+path)
        else:
            base.append("--git-dir="+os.path.join(path, ".git"))
            base.append("--work-tree="+path)
    for arg in args:
        base.append(arg)
    return base

def isclean(path):
    """检查版本库是否是干净的。

    :param str path: git 仓库文件夹路径。
    :rtype: boolean

    """
    return call(path, 'status', '-s')[1] == ''

def get_branches(path):
    """获取当前所有分支名称的列表。

    :param str path: git 仓库文件夹路径。
    :return: 分支名称列表。当前分支位于列表第一项。
    :rtype: list

    """
    code, output = call(path, 'branch', '--list')
    if code > 0:
        return None
    branches = output.split('\n')
    newbr = [None]
    for br in branches:
        if br:
            if br[0] == '*':
                newbr[0] = br[2:]
            else:
                newbr.append(br[2:])
    return newbr
    
def clone(giturl, gitpath):
    """clone 一个 git 库。

    :param str giturl: git 仓库的 url 地址。
    :param str gitpath: git 仓库保存路径。

    """
    gitArgs = ['git', 'clone', giturl, gitpath]
    slog.info(' '.join(gitArgs))
    return subprocess.call(gitArgs)

def get_hash(path, cut=0):
    """获取可被 git 的 HEAD 的 sha1 值。

    :param str path: git 仓库文件夹路径。
    :param int cut: 包含的 sha1 值的长度。0代表不剪切。
    :returns: 剪切过的 sha1 的值。
    :rtype: str

    """
    code, output = call(path, 'rev-parse', 'HEAD')
    if code > 0:
        return None
    # maybe the string is with a linebreak.
    sha1 = output.strip()
    if cut > 0:
        sha1 = sha1[:7]
    return sha1

def get_commit_times(path):
    """获取提交次数。

    :param str path: git 仓库文件夹路径。
    :returns: 包含所有分支的提交次数。
    :rtype: int

    """
    code, output = call(path, "rev-list", '--all')
    if code > 0:
        return None
    return output.count("\n")

def update_submodules(path, init=True, update=True):
    """更新子模块。

    :param str path: git 仓库文件夹路径。
    :param bool init: 是否初始化子模块。
    :param bool update: 是否更新子模块。

    """
    succ = None
    if init:
        arg = get_args(path, 'submodule', 'init')
        slog.info(' '.join(arg))
        succ = subprocess.call(arg)
        if succ>0:
            slog.error('git execute error!')
            return succ
    if update:
        arg = get_args(path, "submodule", "update")
        slog.info(' '.join(arg))
        succ = subprocess.call(arg)
        if succ>0:
            slog.error('git execute error!')
            return succ
    return succ



########################################
# conf.py
# Author zrong(zengrong.net)
#
# Creation 2015-06-14
# Last Editing 2015-06-14
########################################

"""
.. module:: conf
   :platform: Unix, Windows
   :synopsis: 对配置文件的封装。

.. moduleauthor:: zrong(zengrong.net)

"""

import os
import re
from rookout import slog
from rookout.base import (read_file, write_file)
from configparser import (ConfigParser, NoSectionError)

class PYConf(dict):
    """基于 Python dict 的配置文件。

    dict 默认不适合当作配置文件对象使用。如要有下面几点不便：
    
    #. 对于不存在的 key，会 raise KeyError 错误；
    #. dict不能使用 ``.`` 语法访问。

    :class:`rookout.PYConf` 解决了这些问题，还另外提供了一些方法在使用上更加方便。

    """ 

    def __missing__(self, key):
        return None

    def __getattr__(self, name):
        return self[name]

    def __setattr__(self, name, value):
        self[name] = value
        
    def __delattr__(self, name):
        del self[name]

    def copy_from_dict(self, adict, parent=None):
        """从一个已经存在的 dict 中复制所有的值。

        :param adict: 被复制的 dict。
        :type adict: dict
        :param parent:  复制到哪个父对象。
                        若为 None 则复制到 self 。
        :type parent: rookout.PYConf

        """
        if not parent:
            parent = self
        for k,v in adict.items():
            if isinstance(v, dict):
                vDict = PYConf(v)
                self.copy_from_dict(v, vDict)
                parent[k] = vDict
            else:
                parent[k] = v

    def dump(self, human=False):
        """将自身内容打印成字符串

        :param bool human: 若值为 True ，则打印成易读格式。

        """
        txt = str(self)
        if human:
            txt = txt.replace(", '", ",\n'")
            txt = txt.replace("{", "{\n")
            txt = txt.replace("}", "\n}")
            txt = txt.replace("[", "[\n")
            txt = txt.replace("]", "\n]")
        return txt

    def save_to_file(self, path, human=True):
        """将自身内容保存到文件。

        :param str path: 保存的文件路径。
        :param bool human: 参见 :func:`dump()`

        """
        write_file(path, self.dump(human))
        slog.info("Save %a done.", path)

    def read_from_file(self, path):
        """从一个文本文件中读入信息。
        假设该文本文件的格式与 :func:`dump()` 相同。

        :param str path: 待读入的文件路径。

        """
        if not os.path.exists(path):
            slog.warning("The file %s is not exist.", path)
            return False
        txt = read_file(path)
        dic = eval(txt)
        self.copy_from_dict(dic)
        return True

class INIConf(ConfigParser):
    """对 INI 格式的配置文件做了一些变化，使其可以支持列表。

    只需要将 section 的名称设置为 [@list name] 即可加入一个名称为 name 的 section。

    这个 sectioni 中的每一行都作为列表的一项。
    """ 

    _LIST_TMPL = r"""
        @(list)?                            # @ or @list
        \s*(?P<header>\w+)                  # very permissive!
        """
    
    LISTCRE = re.compile(_LIST_TMPL, re.VERBOSE)

    def __init__(self):
        # Only use the '=' as a delimiter
        super().__init__(allow_no_value=True, delimiters=('='))

    # override RawConfigParser to protect case.
    def optionxform(self, optionstr):  
        return optionstr

    def _get_list_name(self, section):
        if not self.LISTCRE.match(section):
            section = "@list "+section
        return section

    def has_list(self, section):
        return self.has_section(self._get_list_name(section))

    def list(self, section):
        return self.options(self._get_list_name(section))

    def lists(self):
        list_sections = []
        for name in self.sections():
            m = self.LISTCRE.search(name)
            if m:
                list_sections.append(m.group('header'))
        return list_sections

def merge_INIConf(a, b):
    """用 b 的内容覆盖 a 的内容（若重名），并返回 a
    """
    for sname in b.sections():
        if a.has_section(sname):
            for oname in b.options(sname):
                a[sname][oname] = b[sname][oname]
        else:
            a[sname] = b[sname]
    return a



########################################
# ftp.py
# Author zrong(zengrong.net)
#
# Creation 2015-02-03
# Last Editing 2015-06-14
########################################

"""
.. module:: ftp
   :platform: Unix, Windows
   :synopsis: 封装 ftplib 的调用。

.. moduleauthor:: zrong(zengrong.net)

"""

import os
import ftplib
from rookout import slog

def get_ftp(ftp_conf, debug=0):
    """得到一个 已经打开的FTP 实例，和一个 ftp 路径。

    :param dict ftp_conf: ftp配置文件，格式如下：
    
        >>> {
        >>>     'server':'127.0.0.1',
        >>>     'start_path':None,
        >>>     'user':'admin',
        >>>     'password':'123456',
        >>> }

    :returns: ftp, ftpserverstr
    :rtype: :class:`ftplib.FTP` , str

    """
    server = ftp_conf.get('server')
    user = ftp_conf.get('user')
    password = ftp_conf.get('password')
    start_path = ftp_conf.get('start_path')
    slog.info("Connecting FTP server %s ......", server)
    ftpStr = 'ftp://%s/'%server
    if start_path:
        ftpStr = ftpStr+start_path
    ftp = ftplib.FTP(server, user, password)
    ftp.set_debuglevel(debug)
    if start_path:
        ftp.cwd(start_path)
    serverFiles = ftp.nlst()
    slog.info('There are some files in %s:\n[%s]'%(ftpStr, ', '.join(serverFiles)))
    return ftp, ftpStr

def upload_file(file_path, remote_path, ftp_conf, remove_file=False):
    """上传第一个指定的文件到 FTP 服务器。

    :param str file_path: 待上传文件的绝对路径。
    :param str remote_path: 文件在 FTP 服务器上的相对路径（相对于 FTP 服务器的初始路径）。
    :param dict ftp_conf: ftp配置文件，详见 :func:`get_ftp` 。
    :param bool remove_file: 上传成功后是否删除本地文件。
    :returns: FTP 服务器上的文件列表
    :rtype: list

    """
    check_ftp_conf(ftp_conf)

    ftp, ftpStr = get_ftp(ftp_conf)
    lf = open(file_path, 'rb')
    slog.info('Uploading "%s" to "%s/%s" ......'%(file_path, ftpStr, remote_path))
    ftp.storbinary("STOR %s"%remote_path, lf)
    filelist = ftp.nlst()
    ftp.quit()
    lf.close()
    if remove_file:
        os.remove(file_path)
    slog.info('Upload done.')
    return filelist

def download_file(remote_path, file_path, ftp_conf):
    check_ftp_conf(ftp_conf)
    ftp, ftpstr = get_ftp(ftp_conf)

    lf = open(file_path, 'wb')
    slog.info('Downloading "%s/%s" to "%s" ......'%(ftpstr, remote_path, lf.name))
    ftp.retrbinary('RETR %s'%remote_path, lf.write)
    ftp.quit()
    lf.close()
    slog.info('Download done.')
    return lf.name

def upload_dir(dir_name, upload_dir, ftp_conf):
    check_ftp_conf(ftp_conf)
    ftp, ftpStr = get_ftp(ftp_conf)
    if dir_name not in ftp.nlst():
        ftp.mkd(dir_name)
    ftp.cwd(dir_name)
    slog.info('Uploading "%s" to "%s/%s" ......'%(upload_dir, ftpStr, dir_name))
    subDirs = ftp.nlst()
    rootLen = len(upload_dir)+1
    for r,d,fl in os.walk(upload_dir):
        if r.split('/')[-1].startswith('.'):
            continue
        for sdir in d:
            if not sdir.startswith('.') \
            and sdir not in subDirs:
                dirPath = os.path.join(r, sdir)[rootLen:]
                ftp.mkd(dirPath)
        for sf in fl:
            filePath = os.path.join(r, sf) 
            f = open(filePath, 'rb')
            ftpPath = filePath[rootLen:]
            slog.info('%s -> %s', filePath, ftpPath)
            ftp.storbinary('STOR %s'%ftpPath, f)
            f.close()
    ftp.quit()

def check_ftp_conf(ftp_conf):
    if not ftp_conf \
    or not ftp_conf.get('server') \
    or not ftp_conf.get('user') \
    or not ftp_conf.get('password'):
        raise KeyError('ftp_conf MUST contains following values:'
                'server,user,password !')



#########################################
# base.py
#
# Author zrong
# Creation 2014-09-23
# Last Editing 2015-06-14
#########################################

"""
.. module:: base
   :platform: Unix, Windows
   :synopsis: 一些通用功能的封装。

.. moduleauthor:: zrong(zengrong.net)

"""

import os
import re
import sys
import zipfile
import shutil
import hashlib
import tempfile
from string import Template
from rookout import slog

def list_dir(sourceDir, include_source=None, include_file=True):
    """与 :func:`os.listdir()` 类似，但提供一些筛选功能，且返回生成器对象。

    :param str sourceDir: 待处理的文件夹。
    :param bool include_source: 遍历结果中是否包含源文件夹的路径。
    :param bool include_file:    是否包含文件。True 表示返回的内容中既包含文件，又
                                包含文件夹；Flase 代表仅包含文件夹。
    :return: 一个生成器对象。

    """
    for cur_file in os.listdir(sourceDir):
        if cur_file.lower() == ".ds_store":
            continue
        pathWithSource = os.path.join(sourceDir, cur_file)
        if include_file or os.path.isdir(pathWithSource):
            if include_source:
                yield pathWithSource
            else:
                yield cur_file

def copy_dir(sou_dir, dst_dir, del_dst=False, del_subdst=False):
    """:func:`shutil.copytree()` 也能实现类似功能，
    但前者要求目标文件夹必须不存在。
    而 copy_dir 没有这个要求，它可以将 sou_dir 中的文件合并到 dst_dir 中。

    :param str sou_dir: 待复制的文件夹；
    :param str dst_dir: 目标文件夹；
    :param bool del_dst: 是否删除目标文件夹。
    :param bool del_subdst: 是否删除目标子文件夹。

    """
    if del_dst and os.path.isdir(del_dst):
        shutil.rmtree(dst_dir)
    os.makedirs(dst_dir, exist_ok=True)
    for cur_file in list_dir(sou_dir):
        dst_file = os.path.join(dst_dir, cur_file)
        cur_file = os.path.join(sou_dir, cur_file)
        if os.path.isdir(cur_file):
            if del_subdst and os.path.isdir(dst_file):
                shutil.rmtree(dst_file)
            os.makedirs(dst_file, exist_ok=True)
            copy_dir(cur_file, dst_file)
        else:
            shutil.copyfile(cur_file, dst_file)

def get_files(path, ext=[], include=True):
    """遍历提供的文件夹的所有子文件夹，饭后生成器对象。

    :param str path: 待处理的文件夹。
    :param list ext: 扩展名列表。
    :param bool include:    若值为 True，代表 ext 提供的是包含列表；
                            否则是排除列表。
    :returns: 一个生成器对象。 

    """
    has_ext = len(ext)>0
    for p, d, fs in os.walk(path):
        for f in fs:
            if has_ext:
                in_ext = False
                for name in ext:
                    if f.endswith(name):
                        in_ext = True
                        break
                if (include and in_ext) or \
                (not include and not in_ext):
                    yield os.path.join(p,f)
            else:
                yield os.path.join(p, f)

def read_file(file_path):
    """读取文本文件的内容。

    :param str file_path: 文件路径。
    :returns: 文件内容。
    :rtype: str

    """
    with open(file_path, mode="r",encoding="utf-8") as afile:
        txt = afile.read()
    return txt

def write_file(file_path, txt):
    """将文本内容写入文件。

    :param str file_path: 文件路径。
    :param str txt: 待写入的文件内容。

    """
    if not os.path.exists(file_path):
        upDir = os.path.dirname(file_path)
        if not os.path.isdir(upDir):
            os.makedirs(upDir)

    with open(file_path, mode="w",encoding="utf-8") as afile:
        afile.write(txt)

def write_by_templ(templ, target, sub_value, safe=False):
    """根据模版写入文件。

    :param str templ: 模版文件所在路径。
    :param str target: 要写入的文件所在路径。
    :param dict sub_value: 被替换的内容。

    """
    templ_txt = read_file(templ)
    txt = None
    if safe:
        txt = Template(templ_txt).safe_substitute(sub_value)
    else:
        txt = Template(templ_txt).substitute(sub_value)
    write_file(target, txt)

def get_md5(path):
    """获取文件的 MD5 值。

    :param str path: 文件路径。
    :returns: MD5 值。
    :rtype: str

    """
    with open(path,'rb') as f:
        md5obj = hashlib.md5()
        md5obj.update(f.read())
        return md5obj.hexdigest()
    raise FileNotFoundError("Error when get md5 for %s!"%path)

def create_zip(files, trim_arcname=None, target_file=None, **zipfile_args):
    """创建一个 zip 文件。

    :param list files: 要创建zip 的文件列表。
    :param int trim_arcname: 若提供这个值，则使用 ZipFile.write(filename, filename[trim_arcname:]) 进行调用。
    :returns: zip 文件的路径。
    :rtype: str

    """
    zipname = None
    azip = None
    if not target_file:
        azip = tempfile.NamedTemporaryFile(mode='wb', delete=False)
        zipname = azip.name
    else:
        azip = target_file
        zipname = target_file.name if hasattr(azip, 'read') else azip
    slog.info('Package %d files to "%s"'%(len(files), azip.name))
    fileNum = len(files)
    curFile = 0
    zipfile_args['mode'] = 'w'
    if not zipfile_args.get('compression'):
        zipfile_args['compression'] = zipfile.ZIP_DEFLATED
    with zipfile.ZipFile(azip, **zipfile_args) as zipf:
       for f in files:
           percent = round(curFile/fileNum*100)
           sys.stdout.write('\r%d%%'%(percent))
           sys.stdout.flush()
           zipf.write(f, f[trim_arcname:] if trim_arcname else None )
           curFile = curFile+1

       sys.stdout.write('\r100%\n')
       sys.stdout.flush()

    if hasattr(azip, 'close'):
        azip.close()
    return zipname

def get_max_ver(fmt, filelist):
    """有一堆字符串，文件名均包含 %d.%d.%d 形式版本号，返回其中版本号最大的那个。
    我一般用它来检测一堆发行版中版本号最大的那个文件。

    :param str fmt: 要检测测字符串形式，例如 rookout-%s.tar.gz ，其中 %s 会被正则替换。
    :param list files: 字符串列表。
    :returns: 版本号最大的字符串。
    :rtype: str

    """
    x, y, z = 0,0,0
    verpat = fmt%'(\d+).(\d+).(\d+)'
    verre = re.compile(r''+verpat+'', re.M) 
    for f in filelist:
        match = verre.search(f)
        if match:
            x1 = int(match.group(1))
            y1 = int(match.group(2))
            z1 = int(match.group(3))
            if x1 >= x and y1 >= y:
                x = x1
                y = y1
                z = z1
    verfmt = fmt%('%d.%d.%d')
    name = verfmt%(x, y, z)
    if x == 0 and y == 0 and z == 0:
        slog.info('Can not find the string "%s" !'%name)
        return None
    return name




# coding: utf-8
#########################################
# __init__.py
#
# Author zrong(zengrong.net)
# Creation 2014-09-23
# Last Editing 2015-06-14
#########################################
"""zrong 编写的基于 python 3.4 的通用功能封装。

.. moduleauthor:: zrong(zengrong.net)

"""

import logging

__version__ = "0.4.0"
__all__ = ['base', 'git', 'lua', 'ftp', 'gettext', 'conf']

slog = logging.getLogger("system")
"""默认的系统 log。"""

__LOG_FMT = logging.Formatter("%(asctime)s %(levelname)-8s %(message)s\n")


# 可改为使用：https://github.com/limodou/uliweb/blob/master/uliweb/utils/coloredlog.py
# 和 https://github.com/tartley/colorama
class ConsoleColor():
	RESET = "\033[0m"
	BOLD = "\033[1m"
	UNDERLINE = "\033[4m"
	BLINK = "\033[5m"
	REVERSE = "\033[7m"
	FADE = "\033[8m"

	BLACK = "\033[30m"
	RED = "\033[31m"
	GREEN = "\033[32m"
	YELLOW = "\033[33m"
	BLUE = "\033[34m"
	MAGENTA = "\033[35m"
	CYAN = "\033[36m"
	WHITE = "\033[37m"

	BG_BLACK = "\033[40m"
	BG_RED = "\033[41m"
	BG_GREEN = "\033[42m"
	BG_YELLOW = "\033[43m"
	BG_BLUE = "\033[44m"
	BG_MAGENTA = "\033[45m"
	BG_CYAN = "\033[46m"
	BG_WHITE = "\033[47m"

        # 黑底彩色
	# B2_BLACK = "\033[90m"
	# B2_RED = "\033[91m"
	# B2_GREEN = "\033[92m"
	# B2_YELLOW = "\033[93m"
	# B2_BLUE = "\033[94m"
	# B2_MAGENTA = "\033[95m"
	# B2_CYAN = "\033[96m"
	# B2_WHITE = "\033[97m"


class ColoredStreamHandler(logging.StreamHandler):
    def format(self, record):
        msg = super(ColoredStreamHandler, self).format(record)
        if record.levelno >= logging.ERROR:
            msg = ConsoleColor.RED+msg+ConsoleColor.RESET
        elif record.levelno == logging.WARNING:
            msg = ConsoleColor.YELLOW+msg+ConsoleColor.RESET
        return msg

def add_log_handler(log, handler=None, debug=None, fmt=None):
    """为一个 :class:`logging.Logger` 的实例增加 handler。

    :param Logger log: 需要处理的 :class:`logging.Logger` 的实例。
    :param Handler handler: 一个 :class:`logging.Handler` 的实例。
    :param int debug: Debug 级别。
    :param str fmt: Handler 的 Formatter。

    """
    if debug:
        log.setLevel(debug)
    if handler:
        # if not fmt:
        #     fmt = __LOG_FMT
        if fmt:
            handler.setFormatter(fmt)
        log.addHandler(handler)



from setuptools import setup, find_packages

setup(
    name = "rookout",
    version = "0.4.4",
    description = "A rookie's workout library by zrong.",
    author = "zrong",
    author_email = "zrongzrong@gmail.com",
    url = "http://github.com/zrong/python",
    license = "MIT",
    keywords = "utils development zrong rookie workout",
    packages = find_packages(exclude=["tests*","docs"]),
    classifiers = [
        "Programming Language :: Python :: 3",
        "Development Status :: 4 - Beta",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Topic :: Software Development :: Libraries :: Python Modules",
        ],
)



#########################################
# gettext.py
#
# Author zrong(zengrong.net)
# Creation 2015-03-04
# Last Editing 2015-06-14
#########################################

"""
.. module:: gettext
   :platform: Unix, Windows
   :synopsis: 调用外部的 gettext 工具链。

.. moduleauthor:: zrong(zengrong.net)

"""

import os
import subprocess
import tempfile
from rookout import slog

class Gettext(object):
    """创建一个对象保存 gettext 的相关配置。

    """ 
    def __init__(self, is_windows=False, binpath=None):
        if is_windows:
            self._xgettext = os.path.join(binpath, "xgettext.exe")
            self._msgfmt = os.path.join(binpath, "msgfmt.exe")
            self._msgmerge = os.path.join(binpath, "msgmerge.exe")
        else:
            self._xgettext = "xgettext"
            self._msgfmt = "msgfmt"
            self._msgmerge = "msgmerge"

    def merge(self, po_file, source_files):
        """从源码中获取所有条目，合并到 po_file 中。

        :param string po_file: 待写入的 po 文件路径。
        :param list source_files :  所有待处理的原文件路径 list。

        """
        # Create a temporary file to write pot file
        pot_file = tempfile.NamedTemporaryFile(mode='wb', prefix='rookout_', delete=False)
        pot_filename = pot_file.name
        slog.info('Create POT file [%s].', pot_filename)
        xargs = [self._xgettext,
                "--package-name=main",
                "--package-version=0.1",
                "--default-domain=main",
                "--from-code=UTF-8",
                "-C", "-k_",
                "--output", pot_filename]
        txt = subprocess.check_output(xargs+source_files, 
                stderr=subprocess.STDOUT, 
                universal_newlines=True)
        if len(txt) > 0:
            raise(ChildProcessError(txt))
        slog.info('Start merge [%s] to [%s].', pot_filename, po_file)
        xargs = [self._msgmerge, "-U", po_file, pot_filename]
        txt = subprocess.check_output(xargs, universal_newlines=True)
        slog.info(txt)
        pot_file.close()
        os.remove(pot_filename)

    def fmt(self, po_file, mo_file):
        """将 po 文件转换成 mo 文件。

        :param string po_file: 待转换的 po 文件路径。
        :param string mo_file: 目标 mo 文件的路径。

        """
        if not os.path.exists(po_file):
            slog.error('The PO file [%s] is non-existen!'%po_file)
            return
        txt = subprocess.check_output([self._msgfmt, 
            '--check', "--strict", '--verbose', 
            "--output-file", mo_file, po_file], 
                stderr=subprocess.STDOUT, 
                universal_newlines=True)
        slog.info(txt)

